<template>
	<div class='widget-wrapper' :style='widgetWrapperStyle'>
        <div class="widget" :style='transitionStyle'>
			<div
				v-bind:key="index"
				v-for='(object, index) in textInfo'
				class='text-wrapper'
				:class='{started: object.start <= time && time < object.end, ended: time >= object.end}'
			>
				<span v-html="object.text" :style='object === currentText ? spanStyle : {backgroundImage: "linear-gradient(var(--accent-color) 0% 100%)"}'/>

				<div class='shadow-wrapper'>
					<span class='shadow' v-html="object.text" :style='object === currentText ? spanStyle : {backgroundImage: "linear-gradient(var(--accent-color) 0% 100%)"}'/>
				</div>

				<div class='background'>
					<span v-html="object.text" />

					<svg style="visibility: hidden; position: absolute;" width="0" height="0" xmlns="http://www.w3.org/2000/svg" version="1.1">
						<defs>
							<filter id="goo"><feGaussianBlur in="SourceGraphic" stdDeviation="4" result="blur" />
								<feColorMatrix in="blur" mode="matrix" values="1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 19 -9" result="goo" />
								<feComposite in="SourceGraphic" in2="goo" operator="atop"/>
							</filter>
						</defs>
					</svg>
				</div>
			</div>
        </div>
	</div>
</template>

<script>
import {positionMixin} from '../helpers/widgets.mjs'

export default {
	mixins: [positionMixin],

	data() {
		return {
			splittedText: '',
		}
	},

	props: {
		text: {
			type: String,
			default: '...',
		},
		time: {
			type: Number,
			default: 0,
		},
		contentItem: {
			type: Object,
		},
	},

	created() {
		this.splittedText = this.text.split('.').filter(
			(item) => Boolean(item.length),
		).map(
			(item) => item + '.',
		)

		const totalLength = this.splittedText.map(
			(item) => item.length,
		).reduce(
			(a, b) => a + b, 0,
		)

		const textInfo = []
		let currentStart = this.contentItem.start

		for (const text of this.splittedText) {
			const duration = (text.length / totalLength) * this.contentItem.duration

			textInfo.push({
				text: text,
				start: currentStart,
				duration,
				end: currentStart + duration,
			})

			currentStart += duration
		}

		this.textInfo = textInfo
	},

	computed: {
		spanStyle() {
			const text = this.currentText
			const phase = (this.time - text.start) / (text.duration)
			const percent = phase * 100 + '%'

			const gradient = `linear-gradient(to right, var(--accent-color) 0 ${percent}, white ${percent} 100%)`

			return {
				backgroundImage: gradient,
			}
		},

		currentText() {
			return this.textInfo.find((object) => object.start <= this.time && this.time < object.end)
		},
	},
}
</script>

<style scoped>

.widget {
	--accent-color: #FFBB00;

	position: absolute;
    bottom: calc(0.65em + 20px);
    left: calc(6% + 110px);
    width: calc(63%);

	font-family: var(--primary-font);
	font-size: .7em;
}

span {
	color: transparent;
	/* text-shadow: rgba(0, 0, 0, .2) .1em .1em 0; */
	background-repeat: no-repeat;
	background-clip: text;
	-webkit-background-clip: text;
	transition: background .1s linear;
}

.background {
	position: absolute;
    top: 0;
	right: 0;
    left: -0.85em;
    z-index: -1;
	opacity: 0.3;
	width: calc(100% + 1.7em);
}

.background span {
	background: #000;
	display: inline;
	filter: url('#goo');
	-webkit-box-decoration-break: clone;
	box-decoration-break: clone;
	padding: 0.65em 0.85em;
}

.shadow-wrapper {
	position: absolute;
	z-index: -1;
	left: .07em;
	top: .07em;
}

.shadow {
	filter: brightness(.5);
	opacity: .3;
}

.text-wrapper {
	width: 100%;

	position: absolute;
	bottom: 1em;
	opacity: 0;

	transition: bottom 1s ease, opacity 0.7s ease, top 1s ease
}

.text-wrapper.started {
	opacity: 1;
	bottom: 0;

	transition: bottom 1s ease, opacity 0.5s ease, top 1s ease
}

.text-wrapper.started .shadow-wrapper::after {
	opacity: 1;
}

.text-wrapper.ended {
	opacity: 0;
	bottom: -.7em;
}

.text-wrapper.ended .shadow-wrapper::after {
	opacity: 0;
	transition-delay: 0s;
	transition-duration: 0s;
}

</style>
